<template>
  <el-bus-crud ref="crud" v-bind="tableConfig" />
</template>

<script>
import { getSortConfig } from '@/utils/form-item-config'
export default {
  data() {
    return {
      tableConfig: {
        url: 'flower/api/flower/list',
        hasNew: false,
        persistSelection: true,
        operationAttrs: {
          width: '170px',
          fixed: 'right',
        },
        onResetEdit: (row) => {
          this.$router.push(`${this.$route.path}/edit/${row.id}`)
        },
        onResetView: (row) => {
          this.$router.push(`${this.$route.path}/view/${row.id}`)
        },
        columns: [
          { type: 'selection' },
          { label: '商品名称', prop: 'name', minWidth: 150, fixed: 'left' },
          {
            label: '商品封面',
            formatter: (row) =>
              row.cover ? (
                <el-bus-image
                  src={row.cover}
                  lazy={true}
                  style="width:50px;height:50px"
                ></el-bus-image>
              ) : null,
            minWidth: 100,
          },
          { label: '级别', prop: 'levelStr', minWidth: 80 },
          { label: '颜色', prop: 'color', minWidth: 100 },
          { label: '供应商名称', prop: 'supplierName', minWidth: 150 },
          { label: '供应商类型', prop: 'supplierType', minWidth: 150 },
          { label: '商品售价', prop: 'price', minWidth: 120 },
          { label: '虚拟销量', prop: 'sales', minWidth: 120 },
          { label: '真实销量', prop: 'realSales', minWidth: 120 },
          { label: '库存', prop: 'stock', minWidth: 120 },
          { label: '限购数量', prop: 'limited', minWidth: 120 },
          { label: '商品状态', prop: 'statusStr', minWidth: 80 },
          { label: '商品标签', prop: 'tags', minWidth: 150 },
          { label: '所属专区', prop: 'zoneName', minWidth: 150 },
          {
            label: '是否推荐',
            formatter: (row) => (row.recommend ? '是' : '否'),
            minWidth: 100,
          },
          { label: '推荐排序', prop: 'recommendRank', minWidth: 120 },
          { label: '提交时间', prop: 'createTime', minWidth: 180 },
          { label: '审核时间', prop: 'auditTime', minWidth: 180 },
          {
            label: '是否显示',
            formatter: (row) => (
              <el-switch
                value={row.shown}
                onChange={this.onShownChange.bind(this, row)}
              ></el-switch>
            ),
            minWidth: 120,
            fixed: 'right',
          },
        ],
        searchFormAttrs: {
          labelWidth: 'auto',
        },
        searchForm: [
          {
            type: 'row',
            items: [
              {
                label: '商品标签：',
                id: 'tags',
                type: 'bus-radio',
                el: {
                  otherInterfaceUri: 'flower/api/flower/tag/list',
                  hasAll: true,
                  props: {
                    label: 'name',
                    value: 'name',
                    dataPath: 'records',
                  },
                  extraQuery: {
                    current: 1,
                    size: 2000,
                  },
                  childType: 'el-radio-button',
                },
                default: '',
                span: 24,
                searchImmediately: true,
              },
              {
                label: '商品状态：',
                id: 'status',
                type: 'bus-radio',
                el: {
                  code: 'FLOWER_STATUS',
                  hasAll: true,
                  childType: 'el-radio-button',
                },
                default: '',
                span: 24,
                searchImmediately: true,
              },
              {
                label: '是否推荐：',
                id: 'recommend',
                type: 'bus-radio',
                el: {
                  fromDict: false,
                  hasAll: true,
                  childType: 'el-radio-button',
                  options: [
                    { label: '是', value: true },
                    { label: '否', value: false },
                  ],
                },
                default: '',
                span: 24,
                searchImmediately: true,
              },
              {
                label: '是否限购：',
                id: 'isLimited',
                type: 'bus-radio',
                el: {
                  fromDict: false,
                  hasAll: true,
                  childType: 'el-radio-button',
                  options: [
                    { label: '是', value: true },
                    { label: '否', value: false },
                  ],
                },
                default: '',
                span: 24,
                searchImmediately: true,
              },
              {
                label: '所属专区：',
                id: 'zoneId',
                type: 'bus-radio',
                el: {
                  otherInterfaceUri: 'flower/api/flower/zone/list',
                  hasAll: true,
                  childType: 'el-radio-button',
                  props: {
                    label: 'name',
                    value: 'id',
                  },
                },
                default: '',
                span: 24,
                searchImmediately: true,
              },
              {
                label: '商品分类：',
                id: 'category',
                component: 'cascader-filter',
                el: {
                  otherInterfaceUri: 'flower/api/flower/category/tree',
                  props: {
                    label: 'name',
                    value: 'id',
                  },
                  emitPath: false,
                  style: 'width:100%',
                  clearable: true,
                  hasAll: true,
                  childType: 'el-radio-button',
                },
                span: 24,
                searchImmediately: true,
              },
              { label: '商品名称：', id: 'name', type: 'input' },
              { label: '供应商：', id: 'supplierName', type: 'input' },
            ],
          },
        ],
        extraButtons: [
          {
            text: '审核',
            show: (row) => row.status === 'PENDING',
            atClick: (row) => {
              this.$refs.crud.$refs.extraDialog[0].show(row)
              return false
            },
          },
          {
            text: '设置推荐排序',
            show: (row) => row.recommend,
            atClick: (row) => {
              this.$refs.crud.$refs.extraDialog[3].show(row)
              return false
            },
          },
        ],
        headerButtons: [
          {
            text: '批量强制下架',
            type: 'primary',
            disabled: (selected) =>
              selected.filter((item) => item.status === 'UP').length === 0,
            atClick: async (selected) => {
              try {
                const selectedGoods = selected.filter(
                  (item) => item.status === 'UP'
                )
                await this.$elBusUtil.confirm(
                  `确定要批量强制下架这${selectedGoods.length}个商品吗？`
                )
                const { code } = await this.$elBusHttp.request(
                  'flower/api/flower/list/off',
                  {
                    method: 'post',
                    data: {
                      ids: selectedGoods.map((item) => item.id),
                    },
                  }
                )
                if (code === 0) {
                  this.$message.success('操作成功')
                  this.$refs.crud.clearSelection()
                } else {
                  return false
                }
              } catch (e) {
                return false
              }
            },
          },
          {
            text: '批量设置标签',
            type: 'primary',
            disabled: (selected) => selected.length === 0,
            atClick: (selected) => {
              this.$refs.crud.$refs.extraDialog[1].show({
                ids: selected.map((item) => item.id),
                tags: selected.length === 1 ? selected[0].tags : null,
              })
              return false
            },
          },
          {
            text: '批量添加专区',
            type: 'primary',
            disabled: (selected) => selected.length === 0,
            atClick: (selected) => {
              this.$refs.crud.$refs.extraDialog[2].show({
                ids: selected.map((item) => item.id),
                zoneId: selected.length === 1 ? selected[0].zoneId : null,
              })
              return false
            },
          },
          {
            text: '批量设置推荐',
            type: 'primary',
            disabled: (selected) =>
              selected.filter((item) => !item.recommend).length === 0,
            atClick: async (selected) => {
              try {
                const selectedGoods = selected.filter((item) => !item.recommend)
                await this.$elBusUtil.confirm(
                  `确定要批量将这${selectedGoods.length}个商品设置为推荐吗？`
                )
                const { code } = await this.$elBusHttp.request(
                  'flower/api/flower/list/recommend/set',
                  {
                    method: 'post',
                    data: {
                      ids: selectedGoods.map((item) => item.id),
                    },
                  }
                )
                if (code === 0) {
                  this.$message.success('操作成功')
                  this.$refs.crud.clearSelection()
                } else {
                  return false
                }
              } catch (e) {
                return false
              }
            },
          },
          {
            text: '批量取消推荐',
            type: 'primary',
            disabled: (selected) =>
              selected.filter((item) => item.recommend).length === 0,
            atClick: async (selected) => {
              try {
                const selectedGoods = selected.filter((item) => item.recommend)
                await this.$elBusUtil.confirm(
                  `确定要批量将这${selectedGoods.length}个商品取消推荐吗？`
                )
                const { code } = await this.$elBusHttp.request(
                  'flower/api/flower/list/recommend/cancel',
                  {
                    method: 'post',
                    data: {
                      ids: selectedGoods.map((item) => item.id),
                    },
                  }
                )
                if (code === 0) {
                  this.$message.success('操作成功')
                  this.$refs.crud.clearSelection()
                } else {
                  return false
                }
              } catch (e) {
                return false
              }
            },
          },
          {
            text: '批量审核',
            type: 'primary',
            disabled: (selected) =>
              selected.filter((item) => item.status === 'PENDING').length === 0,
            atClick: (selected) => {
              this.$refs.crud.$refs.extraDialog[0].show({
                ids: selected
                  .filter((item) => item.status === 'PENDING')
                  .map((item) => item.id),
              })
              return false
            },
          },
          {
            text: '每人限购',
            type: 'primary',
            disabled: (selected) => selected.length === 0,
            atClick: (selected) => {
              this.$refs.crud.$refs.extraDialog[4].show({
                ids: selected.map((item) => item.id),
                limited: selected.length === 1 ? selected[0].limited : undefined,
              })
              return false
            },
          },
        ],
        extraDialogs: [
          {
            title: '审核',
            form: [
              {
                id: 'ids',
                type: 'select',
                el: {
                  multiple: true,
                },
                hidden: () => true,
              },
              {
                label: '审核意见：',
                id: 'result',
                type: 'radio-group',
                options: [
                  { label: '审核通过', value: 'Y' },
                  { label: '审核不通过', value: 'N' },
                ],
                rules: { required: true, message: '请选择审核意见' },
                on: {
                  change: (e, updateForm) => {
                    updateForm({ auditRemarks: '' })
                  },
                },
              },
              {
                label: '不通过原因：',
                id: 'auditRemarks',
                type: 'input',
                el: {
                  type: 'textarea',
                  rows: 6,
                },
                rules: {
                  required: true,
                  message: '请输入不通过原因',
                  trigger: 'blur',
                },
                hidden: (row) => row.result !== 'N',
              },
            ],
            atConfirm: async (val) => {
              let url = ''
              if (Array.isArray(val.ids) && val.ids.length > 0) {
                url =
                  val.result === 'Y'
                    ? 'flower/api/flower/list/pass/multi'
                    : 'flower/api/flower/list/reject/multi'
              } else {
                url =
                  val.result === 'Y'
                    ? 'flower/api/flower/list/pass'
                    : 'flower/api/flower/list/reject'
              }
              const { code } = await this.$elBusHttp.request(url, {
                method: 'post',
                data: val,
              })
              if (code === 0) {
                this.$message.success('操作成功')
                this.$refs.crud.clearSelection()
              }
            },
          },
          {
            title: '批量设置标签',
            form: [
              {
                id: 'ids',
                type: 'select',
                el: {
                  multiple: true,
                },
                hidden: () => true,
              },
              {
                label: '标签：',
                id: 'tags',
                type: 'bus-select',
                el: {
                  interfaceUri: 'flower/api/flower/tag/list',
                  extraQuery: {
                    current: 1,
                    size: 2000,
                  },
                  props: {
                    label: 'name',
                    value: 'name',
                    dataPath: 'records',
                  },
                  filterable: true,
                  multiple: true,
                  style: 'width:100%',
                },
                outputFormat: (val) => {
                  return Array.isArray(val) ? val.join(',') : null
                },
                inputFormat: (row) => {
                  if ('tags' in row) {
                    return row.tags ? row.tags.split(',') : []
                  }
                },
              },
            ],
            atConfirm: async (val) => {
              const { code } = await this.$elBusHttp.request(
                'flower/api/flower/list/tags',
                {
                  method: 'post',
                  data: val,
                }
              )
              if (code === 0) {
                this.$message.success('操作成功')
                this.$refs.crud.clearSelection()
              }
            },
          },
          {
            title: '批量添加专区',
            form: [
              {
                id: 'ids',
                type: 'select',
                el: {
                  multiple: true,
                },
                hidden: () => true,
              },
              {
                label: '专区：',
                id: 'zoneId',
                type: 'bus-select',
                el: {
                  interfaceUri: 'flower/api/flower/zone/list',
                  props: {
                    label: 'name',
                    value: 'id',
                  },
                  filterable: true,
                  multiple: true,
                  style: 'width:100%',
                },
                inputFormat: (row) => {
                  if ('zoneId' in row) {
                    return row.zoneId
                      ? row.zoneId.split(',').map((i) => parseInt(i))
                      : []
                  }
                },
              },
            ],
            atConfirm: async (val) => {
              const { code } = await this.$elBusHttp.request(
                'flower/api/flower/list/zones',
                {
                  method: 'post',
                  data: val,
                }
              )
              if (code === 0) {
                this.$message.success('操作成功')
                this.$refs.crud.clearSelection()
              }
            },
          },
          {
            title: '设置推荐排序',
            form: [
              {
                ...getSortConfig('recommendRank'),
              },
            ],
            atConfirm: async (val) => {
              const { code } = await this.$elBusHttp.request(
                'flower/api/flower/list/recommend/set/rank',
                {
                  method: 'post',
                  data: val,
                }
              )
              if (code === 0) {
                this.$message.success('设置成功')
              }
            },
          },
          {
            title: '每人限购',
            form: [
              {
                id: 'ids',
                type: 'select',
                el: {
                  multiple: true,
                },
                hidden: () => true,
              },
              {
                label: '限购数量：',
                id: 'limited',
                type: 'input-number',
                el: { precision: 0,  min:1, max: 99999999,  style: 'width:100%'},
              },

            ],
            atConfirm: async (val) => {

              const { code } = await this.$elBusHttp.request(
                'flower/api/flower/list/limiteds',
                {
                  method: 'post',
                  data: val,
                }
              )
              if (code === 0) {
                this.$message.success('操作成功')
                this.$refs.crud.clearSelection()
              }
            },
          },
        ],
      },
    }
  },
  head() {
    return {
      title: '商品列表',
    }
  },
  methods: {
    onShownChange(row, e) {
      const url = e
        ? 'flower/api/flower/list/shown'
        : 'flower/api/flower/list/hidden'
      const text = e ? '显示' : '隐藏'
      this.$elBusUtil
        .confirm(`确定要${text}这个商品吗？`)
        .then(async () => {
          const { code } = await this.$elBusHttp.request(url, {
            params: {
              id: row.id,
            },
          })
          if (code === 0) {
            this.$message.success(`${text}成功`)
            this.$refs.crud.getList()
          }
        })
        .catch(() => {})
    },
  },
}
</script>
